"""Module for handling connections to the OWL energy monitor

"""
import logging
import socket
import struct

logger = logging.getLogger(__name__)


class OWLClient(object):
    """ OWL Client for interacting with the OWL energy monitor.

        Args:
            host (str): the host to connect to the network OWL
            port (int): the port to connect to the network OWL
            multi_cast_address (str): the multi-cast address to listen to
            msg_buffer_size (int): the size of the socket message buffer

        Attributes:
            host (str): the host to connect to the network OWL
            port (int): the port to connect to the network OWL
            multi_cast_address (str): the multi-cast address to listen to
            msg_buffer_size (int): the size of the socket message buffer
            socket (obj): the socket for receiving data from the network OWL
        """

    def __init__(self, host='localhost', port=22600,
                 multi_cast_address='224.192.32.19', msg_buffer_size=512):

        self.host = host
        self.port = port
        self.multi_cast_address = multi_cast_address
        self.msg_buffer_size = msg_buffer_size
        self.socket = None

    def initialise_socket(self):
        """Public method to create and open a socket to the network OWL

        Returns:
            None
        """
        self.socket = self._create_socket()

    def get_reading(self):
        """Get an energy consumption reading

        This function reads data from the multi-cast socket and decodes it as
        utf-8.

        Returns:
            reading (str): xml representation of owl energy data
        """
        logger.info("Getting energy reading.")
        if self.socket is not None:
            reading = self.socket.recv(self.msg_buffer_size).decode('utf-8')
            return reading
        else:
            return None

    def _create_socket(self):
        """
        Creates a socket that listens to the OWL multi-cast.

        Creates and configures a socket that can listen to an OWL multi-cast.
        This uses the struct package to interact with the underlying C could to
        supply options.

        Returns:
            A socket that listens to the OWL multi-cast.
        """
        logger.info("Creating OWL socket.")
        # Create a new socket
        owl_socket = socket.socket(family=socket.AF_INET,
                                   type=socket.SOCK_DGRAM,
                                   proto=socket.IPPROTO_UDP)

        # Set the socket options
        owl_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

        # Bind the socket to the local address and port
        owl_socket.bind((self.host, self.port))

        # Convert str to 4sl struct
        multi_req = struct.pack("4sl",
                                socket.inet_aton(self.multi_cast_address),
                                socket.INADDR_ANY)

        # Set socket options
        owl_socket.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP,
                              multi_req)

        return owl_socket

    def _destroy_socket(self):
        """Safely close the multi cast socket

        Returns:
            None
        """

        if self.socket is not None:
            self.socket.close()
            self.socket = None



"""

"""
import logging
import re


logger = logging.getLogger(__name__)


class OWLDevice(object):
    def __init__(self, owl_id=None, last_update=None, signal_strength=None,
                 link_quality=None, battery_level=None, xml_string=None):
        """

        Args:
            owl_id (str):
            last_update (long):
            signal_strength (float):
            link_quality (float):
            battery_level (float):
            xml_string (str):
        """
        self.owl_id = owl_id
        self.last_update = last_update
        self.signal_strength = signal_strength
        self.battery_level = battery_level
        self.link_quality = link_quality

        if xml_string is not None:
            self.process_xml(xml_string)

    def process_xml(self, raw_data):
        """Read the device attributes from the OWL message buffer.

        """
        pattern = (r"<electricity id='(.*)'><timestamp>([0-9]*)<.*"
                   r".*rssi='(-[0-9]+)' lqi='([0-9]+)'.*"
                   r"level='([0-9]+)%.*")

        logger.info("Processing device data.")

        try:
            result = re.match(pattern, raw_data)

            if result is not None:
                self.owl_id = result.group(1)
                self.last_update = int(result.group(2))
                logger.debug("Timestamp: %f", self.last_update)
                self.signal_strength = float(result.group(3))
                logger.debug("Signal strength: %f", self.signal_strength)
                self.link_quality = float(result.group(4))
                logger.debug("Link quality: %f", self.link_quality)
                self.battery_level = float(result.group(5))
                logger.debug("Battery level: %f", self.battery_level)
            else:
                logger.warning("No device data available")
        except TypeError as error:
            logger.error(error)


class OWLEnergyReading(object):
    def __init__(self, channel=0, owl_id=None, current=None, total_current=None,
                 xml_string=None):
        """

        Args:
            channel (int):
            owl_id (str):
            current (float):
            total_current (float):
            xml_string (str):
        """
        self.current = current
        self.total_current = total_current
        self.channel = channel
        self.owl_id = owl_id

        if xml_string is not None:
            self.process_xml(xml_string)

    def process_xml(self, raw_data):
        """Extracts data from the OWL data buffer that matches the pattern.

        """
        pattern = (r"<electricity id='(.*)'><timestamp>.*chan id='({})'>"
                   r"<curr units='w'>([0-9]+\.[0-9]+)</curr>"
                   r"<day units='wh'>([0-9]+\.[0-9]+)"
                   r"</day.*").format(self.channel)

        try:
            result = re.match(pattern, raw_data)

            if result is not None:
                self.owl_id = result.group(1)
                self.channel = int(result.group(2))
                self.current = float(result.group(3))
                logger.debug("current: %f", self.current)
                self.total_current = float(result.group(4))
                logger.debug("total current: %f", self.total_current)
            else:
                logger.warning("No data available on channel %s", self.channel)

        except TypeError as error:
            logger.error(error)



from setuptools import setup
import re

owlinit = open('owlenergy/__init__.py').read()
author = re.search("__author__ = '([^']+)'", owlinit).group(1)
author_email = re.search("__author_email__ = '([^']+)'", owlinit).group(1)
version = re.search("__version__ = '([^']+)'", owlinit).group(1)

setup(
    name='owl-energy',
    version=version,
    long_description=open('README.md').read(),
    packages=['owlenergy'],
    url='https://stretchproject.org',
    license='MIT',
    author=author,
    author_email=author_email,
    description='A python package for OWL energy monitor using sockets',
    keywords='owl energy monitor consumption current',
)
